(define (domain woodworking)
  (:requirements :typing :action-costs)
  (:types
          acolour - object
          awood - object
          woodobj - object
          machine - object
          surface - object
          treatmentstatus - object
          aboardsize - object
          apartsize - object
          highspeed-saw - machine
          glazer - machine
          grinder - machine
          immersion-varnisher - machine
          planer - machine
          saw - machine
          spray-varnisher - machine
          board - woodobj
          part - woodobj
  )
  (:predicates
          (unused ?obj - part)
          (available ?obj - woodobj)
          (surface-condition ?obj - woodobj ?surface - surface)
          (treatment ?obj - part ?treatment - treatmentstatus)
          (colour ?obj - part ?colour - acolour)
          (wood ?obj - woodobj ?wood - awood)
          (boardsize ?board - board ?size - aboardsize)
          (goalsize ?part - part ?size - apartsize)
          (boardsize-successor ?size1 - aboardsize ?size2 - aboardsize)
          (in-highspeed-saw ?b - board ?m - highspeed-saw)
          (empty ?m - highspeed-saw)
          (has-colour ?machine - machine ?colour - acolour)
          (contains-part ?b - board ?p - part)
          (grind-treatment-change ?old - treatmentstatus ?new - treatmentstatus)
          (is-smooth ?surface - surface)
          (is_constant_verysmooth ?x - surface)
          (is_constant_smooth ?x - surface)
          (is_constant_rough ?x - surface)
          (is_constant_varnished ?x - treatmentstatus)
          (is_constant_glazed ?x - treatmentstatus)
          (is_constant_untreated ?x - treatmentstatus)
          (is_constant_colourfragments ?x - treatmentstatus)
          (is_constant_natural ?x - acolour)
          (is_constant_small ?x - apartsize)
          (is_constant_medium ?x - apartsize)
          (is_constant_large ?x - apartsize)
          )
  (:functions
          (total-cost ) - number
          (spray-varnish-cost ?obj - part) - number
          (glaze-cost ?obj - part) - number
          (grind-cost ?obj - part) - number
          (plane-cost ?obj - part) - number
          )
  (:action do-immersion-varnish
          :parameters (
               ?x - part
               ?m - immersion-varnisher
               ?newcolour - acolour
               ?surface - surface
               ?const0 - treatmentstatus
               ?const1 - acolour
               ?const2 - treatmentstatus
               )
          :precondition
               (and
                    (available ?x)
                    (has-colour ?m ?newcolour)
                    (surface-condition ?x ?surface)
                    (is-smooth ?surface)
                    (treatment ?x ?const2)
                    (is_constant_varnished ?const0)
                    (is_constant_natural ?const1)
                    (is_constant_untreated ?const2)
               )
          :effect
               (and
                    (increase (total-cost) 10)
                    (not (treatment ?x ?const2))
                    (treatment ?x ?const0)
                    (not (colour ?x ?const1))
                    (colour ?x ?newcolour)
               )
  )
  (:action do-spray-varnish
          :parameters (
               ?x - part
               ?m - spray-varnisher
               ?newcolour - acolour
               ?surface - surface
               ?const0 - treatmentstatus
               ?const1 - acolour
               ?const2 - treatmentstatus
               )
          :precondition
               (and
                    (available ?x)
                    (has-colour ?m ?newcolour)
                    (surface-condition ?x ?surface)
                    (is-smooth ?surface)
                    (treatment ?x ?const2)
                    (is_constant_varnished ?const0)
                    (is_constant_natural ?const1)
                    (is_constant_untreated ?const2)
               )
          :effect
               (and
                    (increase (total-cost) (spray-varnish-cost ?x))
                    (not (treatment ?x ?const2))
                    (treatment ?x ?const0)
                    (not (colour ?x ?const1))
                    (colour ?x ?newcolour)
               )
  )
  (:action do-glaze
          :parameters (
               ?x - part
               ?m - glazer
               ?newcolour - acolour
               ?const0 - treatmentstatus
               ?const1 - acolour
               ?const2 - treatmentstatus
               )
          :precondition
               (and
                    (available ?x)
                    (has-colour ?m ?newcolour)
                    (treatment ?x ?const2)
                    (is_constant_glazed ?const0)
                    (is_constant_natural ?const1)
                    (is_constant_untreated ?const2)
               )
          :effect
               (and
                    (increase (total-cost) (glaze-cost ?x))
                    (not (treatment ?x ?const2))
                    (treatment ?x ?const0)
                    (not (colour ?x ?const1))
                    (colour ?x ?newcolour)
               )
  )
  (:action do-grind
          :parameters (
               ?x - part
               ?m - grinder
               ?oldsurface - surface
               ?oldcolour - acolour
               ?oldtreatment - treatmentstatus
               ?newtreatment - treatmentstatus
               ?const0 - surface
               ?const1 - acolour
               )
          :precondition
               (and
                    (available ?x)
                    (surface-condition ?x ?oldsurface)
                    (is-smooth ?oldsurface)
                    (colour ?x ?oldcolour)
                    (treatment ?x ?oldtreatment)
                    (grind-treatment-change ?oldtreatment ?newtreatment)
                    (is_constant_natural ?const1)
                    (is_constant_verysmooth ?const0)
               )
          :effect
               (and
                    (increase (total-cost) (grind-cost ?x))
                    (not (surface-condition ?x ?oldsurface))
                    (surface-condition ?x ?const0)
                    (not (treatment ?x ?oldtreatment))
                    (treatment ?x ?newtreatment)
                    (not (colour ?x ?oldcolour))
                    (colour ?x ?const1)
               )
  )
  (:action do-plane
          :parameters (
               ?x - part
               ?m - planer
               ?oldsurface - surface
               ?oldcolour - acolour
               ?oldtreatment - treatmentstatus
               ?const0 - surface
               ?const1 - acolour
               ?const2 - treatmentstatus
               )
          :precondition
               (and
                    (available ?x)
                    (surface-condition ?x ?oldsurface)
                    (treatment ?x ?oldtreatment)
                    (colour ?x ?oldcolour)
                    (is_constant_natural ?const1)
                    (is_constant_smooth ?const0)
                    (is_constant_untreated ?const2)
               )
          :effect
               (and
                    (increase (total-cost) (plane-cost ?x))
                    (not (surface-condition ?x ?oldsurface))
                    (surface-condition ?x ?const0)
                    (not (treatment ?x ?oldtreatment))
                    (treatment ?x ?const2)
                    (not (colour ?x ?oldcolour))
                    (colour ?x ?const1)
               )
  )
  (:action load-highspeed-saw
          :parameters (
               ?b - board
               ?m - highspeed-saw
               )
          :precondition
               (and
                    (empty ?m)
                    (available ?b)
               )
          :effect
               (and
                    (increase (total-cost) 30)
                    (not (available ?b))
                    (not (empty ?m))
                    (in-highspeed-saw ?b ?m)
               )
  )
  (:action unload-highspeed-saw
          :parameters (
               ?b - board
               ?m - highspeed-saw
               )
          :precondition
               (and
                    (in-highspeed-saw ?b ?m)
               )
          :effect
               (and
                    (increase (total-cost) 10)
                    (available ?b)
                    (not (in-highspeed-saw ?b ?m))
                    (empty ?m)
               )
  )
  (:action cut-board-small
          :parameters (
               ?b - board
               ?p - part
               ?m - highspeed-saw
               ?w - awood
               ?surface - surface
               ?size_before - aboardsize
               ?size_after - aboardsize
               ?const0 - apartsize
               ?const1 - acolour
               ?const2 - treatmentstatus
               )
          :precondition
               (and
                    (unused ?p)
                    (goalsize ?p ?const0)
                    (in-highspeed-saw ?b ?m)
                    (wood ?b ?w)
                    (surface-condition ?b ?surface)
                    (boardsize ?b ?size_before)
                    (boardsize-successor ?size_after ?size_before)
                    (is_constant_small ?const0)
                    (is_constant_natural ?const1)
                    (is_constant_untreated ?const2)
               )
          :effect
               (and
                    (increase (total-cost) 10)
                    (not (unused ?p))
                    (available ?p)
                    (wood ?p ?w)
                    (surface-condition ?p ?surface)
                    (colour ?p ?const1)
                    (treatment ?p ?const2)
                    (boardsize ?b ?size_after)
               )
  )
  (:action cut-board-medium
          :parameters (
               ?b - board
               ?p - part
               ?m - highspeed-saw
               ?w - awood
               ?surface - surface
               ?size_before - aboardsize
               ?s1 - aboardsize
               ?size_after - aboardsize
               ?const0 - apartsize
               ?const1 - acolour
               ?const2 - treatmentstatus
               )
          :precondition
               (and
                    (unused ?p)
                    (goalsize ?p ?const0)
                    (in-highspeed-saw ?b ?m)
                    (wood ?b ?w)
                    (surface-condition ?b ?surface)
                    (boardsize ?b ?size_before)
                    (boardsize-successor ?size_after ?s1)
                    (boardsize-successor ?s1 ?size_before)
                    (is_constant_medium ?const0)
                    (is_constant_natural ?const1)
                    (is_constant_untreated ?const2)
               )
          :effect
               (and
                    (increase (total-cost) 10)
                    (not (unused ?p))
                    (available ?p)
                    (wood ?p ?w)
                    (surface-condition ?p ?surface)
                    (colour ?p ?const1)
                    (treatment ?p ?const2)
                    (boardsize ?b ?size_after)
               )
  )
  (:action cut-board-large
          :parameters (
               ?b - board
               ?p - part
               ?m - highspeed-saw
               ?w - awood
               ?surface - surface
               ?size_before - aboardsize
               ?s1 - aboardsize
               ?s2 - aboardsize
               ?size_after - aboardsize
               ?const0 - apartsize
               ?const1 - acolour
               ?const2 - treatmentstatus
               )
          :precondition
               (and
                    (unused ?p)
                    (goalsize ?p ?const0)
                    (in-highspeed-saw ?b ?m)
                    (wood ?b ?w)
                    (surface-condition ?b ?surface)
                    (boardsize ?b ?size_before)
                    (boardsize-successor ?size_after ?s1)
                    (boardsize-successor ?s1 ?s2)
                    (boardsize-successor ?s2 ?size_before)
                    (is_constant_large ?const0)
                    (is_constant_natural ?const1)
                    (is_constant_untreated ?const2)
               )
          :effect
               (and
                    (increase (total-cost) 10)
                    (not (unused ?p))
                    (available ?p)
                    (wood ?p ?w)
                    (surface-condition ?p ?surface)
                    (colour ?p ?const1)
                    (treatment ?p ?const2)
                    (boardsize ?b ?size_after)
               )
  )
  (:action do-saw-small
          :parameters (
               ?b - board
               ?p - part
               ?m - saw
               ?w - awood
               ?surface - surface
               ?size_before - aboardsize
               ?size_after - aboardsize
               ?const0 - apartsize
               ?const1 - acolour
               ?const2 - treatmentstatus
               )
          :precondition
               (and
                    (unused ?p)
                    (goalsize ?p ?const0)
                    (available ?b)
                    (wood ?b ?w)
                    (surface-condition ?b ?surface)
                    (boardsize ?b ?size_before)
                    (boardsize-successor ?size_after ?size_before)
                    (is_constant_small ?const0)
                    (is_constant_natural ?const1)
                    (is_constant_untreated ?const2)
               )
          :effect
               (and
                    (increase (total-cost) 30)
                    (not (unused ?p))
                    (available ?p)
                    (wood ?p ?w)
                    (surface-condition ?p ?surface)
                    (colour ?p ?const1)
                    (treatment ?p ?const2)
                    (boardsize ?b ?size_after)
               )
  )
  (:action do-saw-medium
          :parameters (
               ?b - board
               ?p - part
               ?m - saw
               ?w - awood
               ?surface - surface
               ?size_before - aboardsize
               ?s1 - aboardsize
               ?size_after - aboardsize
               ?const0 - apartsize
               ?const1 - acolour
               ?const2 - treatmentstatus
               )
          :precondition
               (and
                    (unused ?p)
                    (goalsize ?p ?const0)
                    (available ?b)
                    (wood ?b ?w)
                    (surface-condition ?b ?surface)
                    (boardsize ?b ?size_before)
                    (boardsize-successor ?size_after ?s1)
                    (boardsize-successor ?s1 ?size_before)
                    (is_constant_medium ?const0)
                    (is_constant_natural ?const1)
                    (is_constant_untreated ?const2)
               )
          :effect
               (and
                    (increase (total-cost) 30)
                    (not (unused ?p))
                    (available ?p)
                    (wood ?p ?w)
                    (surface-condition ?p ?surface)
                    (colour ?p ?const1)
                    (treatment ?p ?const2)
                    (boardsize ?b ?size_after)
               )
  )
  (:action do-saw-large
          :parameters (
               ?b - board
               ?p - part
               ?m - saw
               ?w - awood
               ?surface - surface
               ?size_before - aboardsize
               ?s1 - aboardsize
               ?s2 - aboardsize
               ?size_after - aboardsize
               ?const0 - apartsize
               ?const1 - acolour
               ?const2 - treatmentstatus
               )
          :precondition
               (and
                    (unused ?p)
                    (goalsize ?p ?const0)
                    (available ?b)
                    (wood ?b ?w)
                    (surface-condition ?b ?surface)
                    (boardsize ?b ?size_before)
                    (boardsize-successor ?size_after ?s1)
                    (boardsize-successor ?s1 ?s2)
                    (boardsize-successor ?s2 ?size_before)
                    (is_constant_large ?const0)
                    (is_constant_natural ?const1)
                    (is_constant_untreated ?const2)
               )
          :effect
               (and
                    (increase (total-cost) 30)
                    (not (unused ?p))
                    (available ?p)
                    (wood ?p ?w)
                    (surface-condition ?p ?surface)
                    (colour ?p ?const1)
                    (treatment ?p ?const2)
                    (boardsize ?b ?size_after)
               )
  )
)
